@@ -5,7 +5,7 @@ import ipywidgets as widgets
 import k3d
 import numpy as np
 from IPython.display import display
-from ipywidgets import Play, Checkbox, IntSlider, Button, ColorPicker, VBox, Tab, HBox, Accordion, Widget
+from ipywidgets import Play, Checkbox, IntSlider, Button, ColorPicker, VBox, Tab, HBox, Accordion, Widget, Label
 from k3d.plot import Plot
 
 from .jointset import JointSet, MuPoTSJoints, OpenPoseJoints, CocoExJoints, PanopticJoints, Common14Joints
@@ -13,19 +13,22 @@ from .loss import L2
 from .objects import Skeleton, DrawableSkeleton, Color, Positions
 
 
+def create_video_player(fps: int, number_of_frames: int) -> Play:
+    return Play(value=0, min=0, max=number_of_frames, step=1,
+                interval=1000 / fps, description='Press play', disabled=False)
+
+
 class VideoPlayer:
     def __init__(self, plot: Plot, fps: int, frames: np.ndarray, video_player: Play = None):
         self.plot: Plot = plot
-        self.video_player: Optional[Play] = video_player if video_player is not None else Play(
-            value=0,
-            min=0, max=frames.shape[0] - 1,
-            step=1, interval=1000 / fps,
-            description='Press play', disabled=False)
+        self.video_player: Optional[Play] =\
+            video_player if video_player is not None else create_video_player(fps, number_of_frames=frames.shape[0] - 1)
         self.fps: int = fps
         self.frames: np.ndarray = frames
 
     def get_video_player_widget(self) -> HBox:
-        frame_slider = IntSlider(value=0, min=0, max=self.frames.shape[0] - 1, step=1, description='Frame')
+        frame_slider = IntSlider(value=self.video_player.value, min=self.video_player.min,
+                                 max=self.video_player.max, step=self.video_player.step, description='Frame')
         next_frame_button = Button(description='Next frame')
         next_frame_button.on_click(self.__update_to_next_frame)
         previous_frame_button = Button(description='Previous frame')
@@ -185,15 +188,111 @@ class ColorChanger:
 
 
 class LossContainer:
-    def __init__(self, pred_skeletons: np.ndarray, gt_skeletons: np.ndarray,
-                 loss: Callable[[np.ndarray, np.ndarray], float]):
-        self.pred_skeletons: np.ndarray = pred_skeletons
-        self.gt_skeletons: np.ndarray = gt_skeletons
-        self.loss: Callable[[np.ndarray, np.ndarray], float] = loss
+    def __init__(self, pred_skeletons: np.ndarray, gt_skeletons: np.ndarray, joint_set: JointSet,
+                 loss: Callable[[np.ndarray, np.ndarray], np.ndarray], video_player: Optional[Play] = None):
+        if video_player is not None:
+            self.is_video: bool = True
+            self.pred_skeletons: np.ndarray = np.swapaxes(pred_skeletons, 0, 1)
+            self.gt_skeletons: np.ndarray = np.swapaxes(gt_skeletons, 0, 1)
+            self.video_player = video_player
+            self.min_loss_frame_index: int = 0
+            self.max_loss_frame_index: int = 0
+        else:
+            self.is_video: bool = False
+            self.pred_skeletons: np.ndarray = np.swapaxes(pred_skeletons[np.newaxis], 0, 1)
+            self.gt_skeletons: np.ndarray = np.swapaxes(gt_skeletons[np.newaxis], 0, 1)
+        self.joint_set: JointSet = joint_set
+        self.number_of_skeletons: int = self.pred_skeletons.shape[0]
+
+        self.joint_losses: np.ndarray = loss(self.pred_skeletons, self.gt_skeletons)
+        self.skeleton_losses = np.sum(self.joint_losses, axis=-1)
+        self.all_losses = np.sum(self.skeleton_losses, axis=0)
+        self.min_skeleton_loss_indices = np.argmin(self.skeleton_losses, axis=0) + 1
+        self.max_skeleton_loss_indices = np.argmax(self.skeleton_losses, axis=0) + 1
+
+        self.skeleton_loss_labels: List[Label] = []
+        self.all_loss_label: Label = Label(value='test')
+        self.min_loss_index_label: Label = Label(value='test')
+        self.max_loss_index_label: Label = Label(value='test')
+
+        self.joint_loss_labels: List[List[Label]] = []
+        self.all_joint_loss_labels: List[Label] = []
+        self.min_joint_loss_name_labels: List[Label] = []
+        self.max_joint_loss_name_labels: List[Label] = []
 
     def get_loss_tab(self) -> Tab:
-        self.loss = lambda x, y: x + y
-        return Tab()
+        loss_tab: Tab = self.__create_empty_loss_tabs()
+        # TODO: fill out losses, link player value to loss indices
+        return loss_tab
+
+    def __create_empty_loss_tabs(self):
+        skeleton_loss_tabs: List[HBox] = []
+        skeleton_losses_column: List[HBox] = []
+        for i in range(self.number_of_skeletons):
+            skeleton_label: Label = Label(value='Skeleton {:d} loss:'.format(i + 1))
+            skeleton_loss: Label = Label(value='test')
+            self.skeleton_loss_labels.append(skeleton_loss)
+            skeleton_losses_column.append(HBox(children=[skeleton_label, skeleton_loss]))
+            joint_loss_labels: List[Label] = []
+            joint_losses_column: List[HBox] = []
+            for j in range(self.joint_set.number_of_joints):
+                joint_label: Label = Label(value=self.joint_set.names[j])
+                joint_loss: Label = Label(value='test')
+                joint_loss_labels.append(joint_loss)
+                joint_losses_column.append(HBox(children=[joint_label, joint_loss]))
+            joint_losses: VBox = VBox(children=joint_losses_column)
+            self.joint_loss_labels.append(joint_loss_labels)
+            skeleton_statistics_column: List[HBox] = []
+            all_joint_loss_label: Label = Label(value='test')
+            self.all_joint_loss_labels.append(all_joint_loss_label)
+            skeleton_statistics_column.append(HBox(children=[Label(value='All losses:'), all_joint_loss_label]))
+            min_loss_joint_label: Label = Label(value='test')
+            self.min_joint_loss_name_labels.append(min_loss_joint_label)
+            skeleton_statistics_column.append(HBox(children=[Label(value='Joint with min loss:'), min_loss_joint_label]))
+            max_loss_joint_label: Label = Label(value='test')
+            self.max_joint_loss_name_labels.append(max_loss_joint_label)
+            skeleton_statistics_column.append(HBox(children=[Label(value='Joint with max loss:'), max_loss_joint_label]))
+            skeleton_statistics: VBox = VBox(children=skeleton_statistics_column)
+            skeleton_loss_tabs.append(HBox(children=[joint_losses, skeleton_statistics]))
+        skeleton_losses: VBox = VBox(children=skeleton_losses_column)
+        statistics_column: List[HBox] = [
+            HBox(children=[Label(value='All losses:'), self.all_loss_label]),
+            HBox(children=[Label(value='Min loss skeleton index:'), self.min_loss_index_label]),
+            HBox(children=[Label(value='Min loss skeleton index:'), self.max_loss_index_label])
+        ]
+        if self.is_video:
+            self.min_loss_frame_index: int = np.argmin(self.all_losses)
+            min_loss_jump_button: Button = Button(description='Min loss frame')
+            min_loss_jump_button.on_click(self.__jump_to_min_loss_frame)
+            self.max_loss_frame_index: int = np.argmax(self.all_losses)
+            max_loss_jump_button: Button = Button(description='Max loss frame')
+            max_loss_jump_button.on_click(self.__jump_to_max_loss_frame)
+            statistics_column.append(HBox(children=[Label(value='Min loss frame index:'),
+                                                    Label(value=str(self.min_loss_frame_index + 1)),
+                                                    min_loss_jump_button]))
+            statistics_column.append(HBox(children=[Label(value='Max loss frame index:'),
+                                                    Label(value=str(self.max_loss_frame_index + 1)),
+                                                    max_loss_jump_button]))
+        statistics: VBox = VBox(children=statistics_column)
+        loss_tabs: List[HBox] = [HBox(children=[skeleton_losses, statistics])] + skeleton_loss_tabs
+        loss_tab = Tab(children=loss_tabs)
+        loss_tab.set_title(0, "All losses")
+        for i in range(len(loss_tabs)):
+            loss_tab.set_title(i + 1, 'Skeleton {:d} losses'.format(i + 1))
+        return loss_tab
+
+    def __jump_to_min_loss_frame(self, button):
+        self.video_player.value = self.min_loss_frame_index
+
+    def __jump_to_max_loss_frame(self, button):
+        self.video_player.value = self.max_loss_frame_index
+
+    def __set_loss_labels(self, index: int) -> None:
+        for i in range(self.number_of_skeletons):
+            pass
+
+    def __update_losses(self, current_frame):
+        a = current_frame.new
 
 
 class SkeletonVisualizer:
@@ -228,14 +327,14 @@ class SkeletonVisualizer:
         color_changer: ColorChanger = ColorChanger()
         color_changer_tab: Tab = color_changer.get_color_changer_widget(self.skeletons)
         self.__display_interface([('Change visibilities', visibility_widget),
-                                  ('Change colors', color_changer_tab)]
-                                 + [] if additional_tabs is None else additional_tabs)
+                                  ('Change colors', color_changer_tab)] +
+                                 ([] if additional_tabs is None else additional_tabs))
 
     def visualize_with_ground_truths(
             self, pred_skeletons: np.ndarray, gt_skeletons: np.ndarray,
             pred_colors: List[Color] = None, gt_colors: List[Color] = None,
             include_names: bool = False, include_coordinates: bool = False,
-            automatic_camera_orientation: bool = False, loss: Callable[[np.ndarray, np.ndarray], float] = L2) -> None:
+            automatic_camera_orientation: bool = False, loss: Callable[[np.ndarray, np.ndarray], np.ndarray] = L2) -> None:
         assert pred_skeletons.shape == gt_skeletons.shape, \
             'The predicate and ground truth skeleton arrays must have the same shape.'
         pred_colors = self.__init_colors(pred_skeletons.shape[0], pred_colors)
@@ -243,7 +342,7 @@ class SkeletonVisualizer:
         skeletons: np.ndarray = np.concatenate((pred_skeletons, gt_skeletons), axis=0)
         colors: List[Color] = pred_colors + gt_colors
         is_gt_list: List[bool] = [False] * len(pred_skeletons) + [True] * len(gt_skeletons)
-        loss_container: LossContainer = LossContainer(pred_skeletons, gt_skeletons, loss)
+        loss_container: LossContainer = LossContainer(pred_skeletons, gt_skeletons, self.joint_set, loss)
         self.visualize(
             skeletons, colors, include_names, include_coordinates, automatic_camera_orientation,
             is_gt_list, additional_tabs=[('Losses', loss_container.get_loss_tab())])
@@ -252,7 +351,7 @@ class SkeletonVisualizer:
             self, frames: np.ndarray, colors: List[Color] = None, fps: int = 15,
             include_names: bool = False, include_coordinates: bool = False,
             automatic_camera_orientation: bool = False, is_gt_list: List[bool] = None,
-            additional_tabs: List[Tuple[str, Widget]] = None) -> None:
+            player: Optional[Play] = None, additional_tabs: List[Tuple[str, Widget]] = None) -> None:
         self.__assert_include_arguments(include_names, include_coordinates)
         assert len(frames.shape) == 4
         first_frame: np.ndarray = frames[0]
@@ -271,43 +370,50 @@ class SkeletonVisualizer:
         self.__link_text_widgets(include_names, include_coordinates)
         self.plot.display()
         visibility_widget: HBox = HBox([self.joint_names_visible, self.joint_coordinates_visible])
-        video_player: VideoPlayer = VideoPlayer(self.plot, fps, frames)
+        video_player: VideoPlayer = VideoPlayer(self.plot, fps, frames, player)
         video_player_widget: HBox = video_player.get_video_player_widget()
         color_changer: ColorChanger = ColorChanger()
         color_changer_tab: Tab = color_changer.get_color_changer_widget(self.skeletons)
         self.__display_interface([('Play video', video_player_widget),
                                   ('Change visibilities', visibility_widget),
-                                  ('Change colors', color_changer_tab)]
-                                 + [] if additional_tabs is None else additional_tabs)
+                                  ('Change colors', color_changer_tab)] +
+                                 ([] if additional_tabs is None else additional_tabs))
 
     def visualize_video_from_file(
             self, file_name: str, colors: List[Color] = None, fps: int = 15,
             include_names: bool = False, include_coordinates: bool = False,
-            automatic_camera_orientation: bool = False) -> None:
+            automatic_camera_orientation: bool = False, is_gt_list: List[bool] = None,
+            player: Optional[Play] = None, additional_tabs: List[Tuple[str, Widget]] = None) -> None:
         file: IO = open(file_name, 'rb')
         frames = pickle.load(file)
         file.close()
         self.visualize_video(
-            frames, colors, fps, include_names, include_coordinates, automatic_camera_orientation)
+            frames, colors, fps, include_names, include_coordinates,
+            automatic_camera_orientation, is_gt_list, player, additional_tabs)
 
     def visualize_video_with_ground_truths(
             self, pred_frames: np.ndarray, gt_frames: np.ndarray,
             pred_colors: List[Color] = None, gt_colors: List[Color] = None,
             fps: int = 15, include_names: bool = False, include_coordinates: bool = False,
-            automatic_camera_orientation: bool = False) -> None:
+            automatic_camera_orientation: bool = False,
+            loss: Callable[[np.ndarray, np.ndarray], np.ndarray] = L2) -> None:
         pred_colors = self.__init_colors(pred_frames.shape[1], pred_colors)
         gt_colors = self.__init_colors(gt_frames.shape[1], gt_colors)
         frames: np.ndarray = np.concatenate((pred_frames, gt_frames), axis=1)
         colors: List[Color] = pred_colors + gt_colors
         is_gt_list: List[bool] = [False] * len(pred_colors) + [True] * len(gt_colors)
+        player: Play = create_video_player(fps, frames.shape[0] - 1)
+        loss_container: LossContainer = LossContainer(pred_frames, gt_frames, self.joint_set, loss, player)
         self.visualize_video(
-            frames, colors, fps, include_names, include_coordinates, automatic_camera_orientation, is_gt_list)
+            frames, colors, fps, include_names, include_coordinates, automatic_camera_orientation,
+            is_gt_list, player, [('Losses', loss_container.get_loss_tab())])
 
     def visualize_video_from_file_with_ground_truths(
             self, pred_file_name: str, gt_file_name: str,
             pred_colors: List[Color] = None, gt_colors: List[Color] = None,
             fps: int = 15, include_names: bool = False, include_coordinates: bool = False,
-            automatic_camera_orientation: bool = False) -> None:
+            automatic_camera_orientation: bool = False,
+            loss: Callable[[np.ndarray, np.ndarray], np.ndarray] = L2) -> None:
         file: IO = open(pred_file_name, 'rb')
         pred_frames = pickle.load(file)
         file.close()
@@ -316,7 +422,7 @@ class SkeletonVisualizer:
         file.close()
         self.visualize_video_with_ground_truths(
             pred_frames, gt_frames, pred_colors, gt_colors, fps,
-            include_names, include_coordinates, automatic_camera_orientation)
+            include_names, include_coordinates, automatic_camera_orientation, loss)
 
     @staticmethod
     def __init_colors(number_of_skeletons: int, colors: List[Color]) -> List[Color]:
@@ -412,7 +518,7 @@ class SkeletonVisualizer:
 
     @staticmethod
     def __display_interface(widget_tuples: List[Tuple[str, Widget]]) -> None:
-        interface: Accordion = Accordion(children=list([widget_tuple[1] for widget_tuple in widget_tuples]))
+        interface: Accordion = Accordion(children=list(map(lambda widget_tuple: widget_tuple[1], widget_tuples)))
         for i in range(len(widget_tuples)):
             interface.set_title(i, widget_tuples[i][0])
         display(interface)
